local LDB = LibStub:GetLibrary("LibDataBroker-1.1")
local LQT = LibStub("LibQTip-1.0")
local abacus = LibStub("LibAbacus-3.0")
local crayon = LibStub("LibCrayon-3.0")
local dataobj = LDB:NewDataObject("Broker_kAddonMsg", {
	type = "data source",
	text = "kAddonMsg",
	icon = "Interface\\AddOns\\Broker_kAddonMsg\\icon.tga",
})
local icon = LibStub("LibDBIcon-1.0")
local ceil = ceil
local floor = floor
local pairs = pairs
local ipairs = ipairs
local strlen = strlen
local table_sort = table.sort
local time = time
local BNSendGameData = BNSendGameData
local SendAddonMessage = SendAddonMessage

Broker_kAddonMsg = LibStub("AceAddon-3.0"):NewAddon("Broker_kAddonMsg", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0")
local self, Broker_kAddonMsg = Broker_kAddonMsg, Broker_kAddonMsg
local tooltip
local defaults = {
	profile = {
		minimap = {
			hide = false,
		},
		filter = {
			realm = {},
			char = {},
		},
	},
	global = {},
}

local function GetOptions(uiType, uiName)
	local options = {
		type = "group",
		name = GetAddOnMetadata("Broker_kAddonMsg", "Title"),
		args = {
			desc = {
				type = "description",
				order = 0,
				name = GetAddOnMetadata("Broker_kAddonMsg", "Notes"),
			},
			minimap = {
				name = "Minimap Icon",
				desc = "Toggle minimap icon",
				type = "toggle",
				order = 10,
				get = function() return not self.db.profile.minimap.hide end,
				set = function()
					self.db.profile.minimap.hide = not self.db.profile.minimap.hide
					if self.db.profile.minimap.hide then
						icon:Hide("Broker_kAddonMsg")
					else
						icon:Show("Broker_kAddonMsg")
					end
				end,
			},
		},
	}
	return options
end

local function Round(value, decimal)
  if (decimal) then
    return floor((value * 10^decimal) + 0.5) / (10^decimal)
  else
    return floor(value+0.5)
  end
end

function Broker_kAddonMsg:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("Broker_kAddonMsgDB", defaults, true)

	-- Minimap Icon
	icon:Register("Broker_kAddonMsg", dataobj, self.db.profile.minimap)

	-- Options
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Broker_kAddonMsg", GetOptions)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Broker_kAddonMsg", GetAddOnMetadata("Broker_kAddonMsg", "Title"))
end

function Broker_kAddonMsg:OnEnable()
	self:Hook('BNSendGameData', 'MessageSent', true)
	self:Hook('SendAddonMessage', 'MessageSent', true)	
	self:RegisterEvent('CHAT_MSG_ADDON', 'MessageReceived')
	self:RegisterEvent('BN_CHAT_MSG_ADDON', 'MessageReceived')

	-- Reset
	self:Reset()
end

function Broker_kAddonMsg:CreateEntry(prefix, message, channel)
	self.db.profile.addons = self.db.profile.addons or {}
	self.db.profile.addons[prefix] = self.db.profile.addons[prefix] or {}
	self.db.profile.addons[prefix].count = (self.db.profile.addons[prefix].count or 0) + 1
	self.db.profile.addons[prefix].length = (self.db.profile.addons[prefix].length or 0) + strlen(message)	
end

function Broker_kAddonMsg:MessageSent(prefix, message, channel)
	--print(('Broker_kAddonMsg:MessageReceived - prefix: %s, message: %s, channel: %s'):format(prefix, message, channel))
	self:CreateEntry(prefix, message, channel)
end

function Broker_kAddonMsg:MessageReceived(event, prefix, message, channel, sender)
	--print(('Broker_kAddonMsg:MessageReceived - event: %s, prefix: %s, message: %s, channel: %s, sender: %s'):format(event, prefix, message, channel, sender))
	self:CreateEntry(prefix, message, channel)
end

function Broker_kAddonMsg:DrawTooltip()
	-- Don't show if we're in combat.
	if InCombatLockdown() then
		return
	end

	tooltip:Hide()
	tooltip:Clear()

	local myFont
	if not Broker_kAddonMsg_Tooltip_Font then
		myFont = CreateFont("Broker_kAddonMsg_Tooltip_Font")
		local filename, size, flags = tooltip:GetFont():GetFont()
		myFont:SetFont(filename, size, flags)
		myFont:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
	else
		myFont = Broker_kAddonMsg_Tooltip_Font
	end
	tooltip:SetFont(myFont)

	-- Header
	tooltip:AddHeader("|cffeda55fkAddonMsg|r")

	local elapsed = time() - self.db.profile.timestamp

	if not self.db.profile.addons then
		tooltip:AddLine(("|cffeda55f%s|r"):format("No addon communication recorded."))
	else
		tooltip:AddHeader("Prefix", "Msgs", "Chars", "per min")
		local totalLength, totalCount = 0, 0
		for i,v in pairs(self.db.profile.addons) do
			tooltip:AddLine(i, v.count, v.length, Round((v.count / elapsed) * 60, 2))
			totalCount = totalCount + v.count
			totalLength = totalLength + v.length
		end
		tooltip:AddHeader("Totals", totalCount, totalLength, Round((totalCount / elapsed) * 60, 2))
	end

	tooltip:AddLine(" ")
	tooltip:AddHeader(("Elapsed: |cffeda55f%s|r"):format(abacus:FormatDurationCondensed(elapsed, false, false)))

	tooltip:AddLine(" ")
	tooltip:AddLine("|cffeda55fClick|r to reset.")
	tooltip:AddLine("|cffeda55fRight-Click|r for options.")

	tooltip:UpdateScrolling()
	tooltip:Show()
end

function Broker_kAddonMsg:Reset()
	self.db.profile.addons = nil
	self.db.profile.timestamp = time()
end

-- LDB functions
function dataobj:OnEnter()
	if not LQT:IsAcquired("Broker_kAddonMsg") then
		tooltip = LQT:Acquire("Broker_kAddonMsgTip",
			-- Columns
			4,
			-- Alignments
			"LEFT", "CENTER", "CENTER", "RIGHT"
		)
	end
	tooltip:Clear()
	tooltip:SmartAnchorTo(self)
	tooltip:SetAutoHideDelay(0.25, self)
	tooltip:SetScale(1)

	Broker_kAddonMsg:DrawTooltip()
end

-- Handled by the AutoHide
--function dataobj:OnLeave()
--	LQT:Release(tooltip)
--	tooltip = nil
--end

function dataobj:OnClick(button)
	if button == "LeftButton" then
		Broker_kAddonMsg:Reset()
	elseif button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata("Broker_kAddonMsg", "Title"))
	end
end
